// Code generated by "stringer -type Location"; DO NOT EDIT.

package stack

import "strconv"

func _() {
	// An "invalid array index" compiler error signifies that the constant values have changed.
	// Re-run the stringer command to generate them again.
	var x [1]struct{}
	_ = x[LocationUnknown-0]
	_ = x[GoMod-1]
	_ = x[GOPATH-2]
	_ = x[GoPkg-3]
	_ = x[Stdlib-4]
	_ = x[lastLocation-5]
}

const _Location_name = "LocationUnknownGoModGOPATHGoPkgStdliblastLocation"

var _Location_index = [...]uint8{0, 15, 20, 26, 31, 37, 49}

func (i Location) String() string {
	if i < 0 || i >= Location(len(_Location_index)-1) {
		return "Location(" + strconv.FormatInt(int64(i), 10) + ")"
	}
	return _Location_name[_Location_index[i]:_Location_index[i+1]]
}
